@@ -1,7 +1,7 @@
-from graphio.objects.nodeset import NodeSet
+from graphio.objects.nodeset import NodeSet, NodeSetDefinition
 from graphio.objects.unstructured_nodeset import UnstructuredNodeSet, Node
 from graphio.objects.unstructured_relationshipset import UnstructuredRelationshipSet, Relationship, NodeMatch
-from graphio.objects.relationshipset import RelationshipSet
+from graphio.objects.relationshipset import RelationshipSet, RelationshipSetDefinition
 from graphio.objects.datacontainer import Container
 from graphio.model import ModelNode, ModelRelationship, MergeKey, Label, NodeDescriptor
 from graphio.objects.properties import ArrayProperty
\ No newline at end of file
