@@ -5,7 +5,9 @@ import json
 import csv
 import gzip
 from collections import defaultdict
-from typing import Set
+from typing import Set, List, Union
+
+from pydantic import BaseModel
 
 from graphio.helper import chunks, create_single_index, create_composite_index
 from graphio import defaults
@@ -22,12 +24,41 @@ CYPHER_TYPE_TO_FUNCTION = {int: 'toInteger',
 TYPE_CONVERSION = {'int': int,
                    'float': float}
 
+
+class NodeSetDefinition(BaseModel):
+    """
+    NodeSetDefinition is a NodeSet without the actual data. Independent class for now, can be parent
+    class for NodeSet in the future.
+    """
+    uuid: str
+    labels: List[str]
+    merge_keys: List[str]
+    default_props: dict = {}
+    preserve: List[str] = []
+    append_props: List[str] = []
+    indexed: bool = False
+    additional_labels: List[str] = []
+    num_nodes: int = 0
+
+    def props(self):
+        """Node properties for NodeSetDefinition."""
+        return {'uuid': self.uuid,
+                'labels': self.labels,
+                'merge_keys': self.merge_keys,
+                'default_props': str(self.default_props),
+                'preserve': self.preserve,
+                'append_props': self.append_props,
+                'indexed': self.indexed,
+                'additional_labels': self.additional_labels,
+                'num_nodes': self.num_nodes}
+
 class NodeSet:
     """
     Container for a set of Nodes with the same labels and the same properties that define uniqueness.
     """
 
-    def __init__(self, labels=None, merge_keys=None, batch_size=None, default_props=None, preserve=None, append_props=None, indexed=False):
+    def __init__(self, labels=None, merge_keys=None, batch_size=None, default_props=None, preserve=None, append_props=None, indexed=False,
+                 additional_labels: List[str]=None):
         """
 
         :param labels: The labels for the nodes in this NodeSet.
@@ -43,6 +74,7 @@ class NodeSet:
         self.preserve = preserve
         self.append_props = append_props
         self.indexed = indexed
+        self.additional_labels = additional_labels
 
         if self.labels:
             self.combined = '_'.join(sorted(self.labels)) + '_' + '_'.join(sorted(self.merge_keys))
@@ -60,6 +92,12 @@ class NodeSet:
         # this works for both unique and non-unique settings
         self.node_index = defaultdict(list)
 
+    def to_definition(self):
+        """Create a NodeSetDefinition from this NodeSet. Later, NodeSetDefinition can become parent class of NodeSet."""
+        return NodeSetDefinition(uuid=self.uuid, labels=self.labels, merge_keys=self.merge_keys, default_props=self.default_props or {},
+                                 preserve=self.preserve or [], append_props=self.append_props or [], indexed=self.indexed,
+                                 additional_labels=self.additional_labels or [], num_nodes=len(self.nodes))
+
     def __str__(self):
         return f"<NodeSet ({self.labels}; {self.merge_keys})>"
 
@@ -349,7 +387,11 @@ class NodeSet:
             batch_size = self.batch_size
         log.debug('Batch Size: {}'.format(batch_size))
 
-        q = nodes_create_unwind(self.labels)
+        if self.additional_labels:
+            all_labels = self.labels + self.additional_labels
+            q = nodes_create_unwind(all_labels)
+        else:
+            q = nodes_create_unwind(self.labels)
 
         for batch in chunks(self.nodes, size=batch_size):
             run_query_return_results(graph, q, database=database, props=list(batch))
@@ -380,25 +422,25 @@ class NodeSet:
         log.debug('Batch Size: {}'.format(batch_size))
 
         if not self.preserve and not self.append_props:
-            q = nodes_merge_unwind(self.labels, self.merge_keys)
+            q = nodes_merge_unwind(self.labels, self.merge_keys, additional_labels=self.additional_labels)
             for batch in chunks(self.node_properties(), size=batch_size):
                 run_query_return_results(graph, q, database=database, props=list(batch))
 
         elif self.preserve and not self.append_props:
-            q = nodes_merge_unwind_preserve(self.labels, self.merge_keys, property_parameter='props')
+            q = nodes_merge_unwind_preserve(self.labels, self.merge_keys, property_parameter='props', additional_labels=self.additional_labels)
             for batch in chunks(self.node_properties(), size=batch_size):
                 run_query_return_results(graph, q, database=database, props=list(batch), preserve=self.preserve)
 
         elif not self.preserve and self.append_props:
             q = nodes_merge_unwind_array_props(self.labels, self.merge_keys, self.append_props,
-                                               property_parameter='props')
+                                               property_parameter='props', additional_labels=self.additional_labels)
             for batch in chunks(self.node_properties(), size=batch_size):
                 run_query_return_results(graph, q, database=database, props=list(batch), append_props=self.append_props)
 
         elif self.preserve and self.append_props:
 
             q = nodes_merge_unwind_preserve_array_props(self.labels, self.merge_keys, self.append_props, self.preserve,
-                                                        property_parameter='props')
+                                                        property_parameter='props', additional_labels=self.additional_labels)
             for batch in chunks(self.node_properties(), size=batch_size):
                 run_query_return_results(graph, q, database=database, props=list(batch), append_props=self.append_props, preserve=self.preserve)
 
